"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildStyledComponent = void 0;
var generator_1 = __importDefault(require("@babel/generator"));
var template_1 = __importDefault(require("@babel/template"));
var t = __importStar(require("@babel/types"));
var css_1 = require("@compiled/css");
var utils_1 = require("@compiled/utils");
var is_prop_valid_1 = __importDefault(require("@emotion/is-prop-valid"));
var constants_1 = require("../constants");
var ast_1 = require("./ast");
var build_css_variables_1 = require("./build-css-variables");
var compress_class_names_for_ax_1 = require("./compress-class-names-for-ax");
var css_builders_1 = require("./css-builders");
var hoist_sheet_1 = require("./hoist-sheet");
var transform_css_items_1 = require("./transform-css-items");
/**
 * Builds up the inline style prop value for a Styled Component.
 *
 * @param variables CSS variables that will be placed in the AST
 */
var styledStyleProp = function (variables) {
    var props = [
        t.spreadElement(t.identifier(constants_1.STYLE_IDENTIFIER_NAME)),
    ];
    return t.objectExpression(props.concat((0, build_css_variables_1.buildCssVariables)(variables, function (node) {
        // Allows us to use component's closure scope instead of arrow function
        return t.isArrowFunctionExpression(node) ? (0, ast_1.pickFunctionBody)(node) : node;
    })));
};
/**
 * Returns a tag string in the form of an identifier or string literal.
 *
 * A type of InBuiltComponent will return a string literal,
 * otherwise an identifier string will be returned.
 *
 * @param tag Made of name and type.
 */
var buildComponentTag = function (_a) {
    var name = _a.name, type = _a.type;
    return type === 'InBuiltComponent' ? "\"".concat(name, "\"") : name;
};
var invalidDomPropsVisitor = {
    MemberExpression: function (path) {
        var _a = path.node, object = _a.object, property = _a.property;
        if (t.isIdentifier(object, { name: constants_1.PROPS_IDENTIFIER_NAME }) && t.isIdentifier(property)) {
            var name_1 = property.name;
            if (name_1 !== 'children' && !(0, is_prop_valid_1.default)(name_1)) {
                this.invalids.add(name_1);
            }
        }
    },
};
/**
 * Finds all prop usage in a component and returns a list
 * of props that are not valid HTML attributes
 *
 * @param path Path of the styled component.
 */
var getInvalidDomProps = function (path) {
    var state = { invalids: new Set() };
    path.traverse(invalidDomPropsVisitor, state);
    return Array.from(state.invalids);
};
/**
 * Will return a generated AST for a Styled Component.
 *
 * @param opts {StyledTemplateOpts} Template options
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var styledTemplate = function (opts, meta) {
    var _a, _b;
    var nonceAttribute = meta.state.opts.nonce ? "nonce={".concat(meta.state.opts.nonce, "}") : '';
    var styleProp = opts.variables.length
        ? styledStyleProp(opts.variables)
        : t.identifier(constants_1.STYLE_IDENTIFIER_NAME);
    var isInBuiltComponent = opts.tag.type === 'InBuiltComponent';
    // This completely depends on meta.parentPath to be the styled component.
    // If this changes please pass the component in another way
    var invalidDomProps = isInBuiltComponent ? getInvalidDomProps(meta.parentPath) : [];
    var hasInvalidDomProps = Boolean(invalidDomProps.length);
    var unconditionalClassNames = '', conditionalClassNames = '';
    opts.classNames.forEach(function (item) {
        if (t.isStringLiteral(item)) {
            unconditionalClassNames += "".concat(item.value, " ");
        }
        else if (t.isLogicalExpression(item) || t.isConditionalExpression(item)) {
            conditionalClassNames += "".concat((0, generator_1.default)(item).code, ", ");
        }
    });
    // Extract the component name from declaration
    // i.e. componentName is `FooBar` given `const FooBar = styled.div(...)`
    var componentName = (_b = (_a = meta.parentPath.parent) === null || _a === void 0 ? void 0 : _a.id) === null || _b === void 0 ? void 0 : _b.name;
    // Add componentClassName if `addComponentName` is enabled and on non-production environment
    var componentClassName = (0, utils_1.toBoolean)(meta.state.opts.addComponentName) &&
        process.env.NODE_ENV !== 'production' &&
        componentName
        ? "\"c_".concat(componentName, "\", ")
        : '';
    var classNames = "".concat(componentClassName, "\"").concat(unconditionalClassNames.trim(), "\", ").concat(conditionalClassNames);
    return (0, template_1.default)("\n  forwardRef(({\n    as: C = ".concat(buildComponentTag(opts.tag), ",\n    style: ").concat(constants_1.STYLE_IDENTIFIER_NAME, ",\n    ...").concat(constants_1.PROPS_IDENTIFIER_NAME, "\n  }, ").concat(constants_1.REF_IDENTIFIER_NAME, ") => {\n    ").concat(hasInvalidDomProps
        ? "const {".concat(invalidDomProps.join(', '), ", ...").concat(constants_1.DOM_PROPS_IDENTIFIER_NAME, "} = ").concat(constants_1.PROPS_IDENTIFIER_NAME, ";")
        : '', "\n\n    return (\n      <CC>\n        <CS ").concat(nonceAttribute, ">{%%cssNode%%}</CS>\n        <C\n          {...").concat(hasInvalidDomProps ? constants_1.DOM_PROPS_IDENTIFIER_NAME : constants_1.PROPS_IDENTIFIER_NAME, "}\n          style={%%styleProp%%}\n          ref={").concat(constants_1.REF_IDENTIFIER_NAME, "}\n          className={ax([").concat(classNames, " ").concat(constants_1.PROPS_IDENTIFIER_NAME, ".className])}\n        />\n      </CC>\n    );\n  });\n"), {
        plugins: ['jsx'],
    })({
        styleProp: styleProp,
        cssNode: t.arrayExpression((0, utils_1.unique)(opts.sheets).map(function (sheet) { return (0, hoist_sheet_1.hoistSheet)(sheet, meta); })),
    });
};
/**
 * Find CSS selectors that are apart of incomplete closures
 * i.e. `:hover {`
 *
 * @param css {string} Template options
 */
var findOpenSelectors = function (css) {
    // Remove any occurrence of { or } inside quotes to stop them
    // interfering with closure matches
    var searchArea = css.replace(/['|"].*[{|}].*['|"]/g, '');
    // Skip over complete closures
    searchArea = searchArea.substring(searchArea.lastIndexOf('}') + 1);
    // Regex for CSS selector
    //[^;\s] Don't match ; or whitespace
    // .+\n?{ Match anything (the selector itself) followed by any newlines then {
    return searchArea.match(/[^;\s].+\n?{/g);
};
/**
 * Returns a Styled Component AST.
 *
 * @param tag {Tag} Styled tag either an inbuilt or user define
 * @param cssOutput {CSSOutput} CSS and variables to place onto the component
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var buildStyledComponent = function (tag, cssOutput, meta) {
    var unconditionalCss = '';
    var conditionalCssItems = [];
    cssOutput.css.forEach(function (item) {
        if (item.type === 'logical' || item.type === 'conditional') {
            // TODO: Optimize this to only run if there is a
            // potential selector scope change
            var selectors = findOpenSelectors(unconditionalCss);
            if (selectors) {
                (0, transform_css_items_1.applySelectors)(item, selectors);
            }
            conditionalCssItems.push(item);
        }
        else {
            unconditionalCss += (0, css_builders_1.getItemCss)(item);
        }
    });
    // Rely on transformCss to remove duplicates and return only the last unconditional CSS for each property
    var uniqueUnconditionalCssOutput = (0, css_1.transformCss)(unconditionalCss, meta.state.opts);
    // Rely on transformItemCss to build expressions for conditional & logical CSS
    var conditionalCssOutput = (0, transform_css_items_1.transformCssItems)(conditionalCssItems, meta);
    var sheets = __spreadArray(__spreadArray([], uniqueUnconditionalCssOutput.sheets, true), conditionalCssOutput.sheets, true);
    var classNames = __spreadArray(__spreadArray([], [
        t.stringLiteral((0, compress_class_names_for_ax_1.compressClassNamesForAx)(uniqueUnconditionalCssOutput.classNames, meta.state.opts.classNameCompressionMap).join(' ')),
    ], false), conditionalCssOutput.classNames, true);
    return styledTemplate({
        classNames: classNames,
        tag: tag,
        sheets: sheets,
        variables: cssOutput.variables,
    }, meta);
};
exports.buildStyledComponent = buildStyledComponent;
//# sourceMappingURL=build-styled-component.js.map